@@ -83,11 +83,11 @@ the function is called with the same input parameters the result is retrieved
 from the cache and not recomputed. There are many implementations of 
 ``memoize`` in http://www.python.org/moin/PythonDecoratorLibrary, 
 but they do not preserve the signature.
-A simple implementation for Python 2.5 could be the following (notice
+A simple implementation could be the following (notice
 that in general it is impossible to memoize correctly something
 that depends on non-hashable arguments):
 
-$$memoize25
+$$memoize_uw
 
 Here we used the functools.update_wrapper_ utility, which has
 been added in Python 2.5 expressly to simplify the definition of decorators
@@ -100,14 +100,14 @@ from the original function to the decorated function by hand).
 The implementation above works in the sense that the decorator 
 can accept functions with generic signatures; unfortunately this
 implementation does *not* define a signature-preserving decorator, since in
-general ``memoize25`` returns a function with a
+general ``memoize_uw`` returns a function with a
 *different signature* from the original function.
 
 Consider for instance the following case:
 
 .. code-block:: python
 
- >>> @memoize25
+ >>> @memoize_uw
  ... def f1(x):
  ...     time.sleep(1) # simulate some long computation
  ...     return x
@@ -119,8 +119,8 @@ keyword arguments:
 .. code-block:: python
 
  >>> from inspect import getargspec 
- >>> print getargspec(f1) 
- ([], 'args', 'kw', None)
+ >>> print getargspec(f1) # I am using Python 2.6+ here
+ ArgSpec(args=[], varargs='args', keywords='kw', defaults=None)
 
 This means that introspection tools such as pydoc will give
 wrong informations about the signature of ``f1``. This is pretty bad:
@@ -160,7 +160,7 @@ At this point you can define your decorator as follows:
 
 $$memoize
 
-The difference with respect to the Python 2.5 approach, which is based
+The difference with respect to the ``memoize_uw`` approach, which is based
 on nested functions, is that the decorator module forces you to lift
 the inner function at the outer level (*flat is better than nested*).
 Moreover, you are forced to pass explicitly the function you want to
@@ -186,7 +186,7 @@ The signature of ``heavy_computation`` is the one you would expect:
 .. code-block:: python
 
  >>> print getargspec(heavy_computation) 
- ([], None, None, None)
+ ArgSpec(args=[], varargs=None, keywords=None, defaults=None)
 
 A ``trace`` decorator
 ------------------------------------------------------
@@ -219,7 +219,7 @@ and it that it has the correct signature:
 .. code-block:: python
 
  >>> print getargspec(f1) 
- (['x'], None, None, None)
+ ArgSpec(args=['x'], varargs=None, keywords=None, defaults=None)
 
 The same decorator works with functions of any signature:
 
@@ -233,7 +233,7 @@ The same decorator works with functions of any signature:
  calling f with args (0, 3, 2), {}
  
  >>> print getargspec(f) 
- (['x', 'y', 'z'], 'args', 'kw', (1, 2))
+ ArgSpec(args=['x', 'y', 'z'], varargs='args', keywords='kw', defaults=(1, 2))
 
 That includes even functions with exotic signatures like the following:
 
@@ -243,7 +243,7 @@ That includes even functions with exotic signatures like the following:
  ... def exotic_signature((x, y)=(1,2)): return x+y
  
  >>> print getargspec(exotic_signature)
- ([['x', 'y']], None, None, ((1, 2),))
+ ArgSpec(args=[['x', 'y']], varargs=None, keywords=None, defaults=((1, 2),))
  >>> exotic_signature() 
  calling exotic_signature with args ((1, 2),), {}
  3
@@ -297,49 +297,6 @@ If you are using an old Python version (Python 2.4) the
 ``decorator`` module provides a poor man replacement for
 ``functools.partial``.
 
-There is also an easy way to create one-parameter factories of
-decorators, based on the following
-``decorator_factory`` utility:
-
-$$decorator_factory
-
-``decorator_factory`` converts a function with signature
-``(param, func, *args, **kw)`` into a one-parameter family
-of decorators. Suppose for instance you want to generated different
-tracing generator, with different tracing messages.
-Here is how to do it:
-
-.. code-block:: python
-
- >>> @decorator_factory
- ... def trace_factory(message_template, f, *args, **kw):
- ...     name = f.func_name
- ...     print message_template % locals()
- ...     return f(*args, **kw)
-
-.. code-block:: python
-
- >>> trace_factory # doctest: +ELLIPSIS
- <functools.partial object at 0x...>
-
- >>> trace = trace_factory('Calling %(name)s with args %(args)s '
- ...                        'and keywords %(kw)s')
-
-In this example the parameter (``message_template``) is 
-just a string, but in general it can be a tuple, a dictionary, or
-a generic object, so there is no real restriction (for instance,
-if you want to define a two-parameter family of decorators just
-use a tuple with two arguments as parameter).
-Here is an example of usage:
-
-.. code-block:: python
-
- >>> @trace
- ... def func(): pass
-
- >>> func()
- Calling func with args () and keywords {}
-
 ``blocking``
 -------------------------------------------
 
@@ -427,18 +384,20 @@ be locked. Here is a minimalistic example:
 Each call to ``write`` will create a new writer thread, but there will 
 be no synchronization problems since ``write`` is locked.
 
->>> write("data1") # doctest: +ELLIPSIS
-<Thread(write-1, started...)>
+.. code-block:: python
 
->>> time.sleep(.1) # wait a bit, so we are sure data2 is written after data1
-
->>> write("data2") # doctest: +ELLIPSIS
-<Thread(write-2, started...)>
-
->>> time.sleep(2) # wait for the writers to complete
-
->>> print datalist
-['data1', 'data2']
+ >>> write("data1") # doctest: +ELLIPSIS
+ <Thread(write-1, started...)>
+ 
+ >>> time.sleep(.1) # wait a bit, so we are sure data2 is written after data1
+ 
+ >>> write("data2") # doctest: +ELLIPSIS
+ <Thread(write-2, started...)>
+ 
+ >>> time.sleep(2) # wait for the writers to complete
+ 
+ >>> print datalist
+ ['data1', 'data2']
 
 The ``FunctionMaker`` class
 ---------------------------------------------------------------
@@ -772,55 +731,56 @@ a *copy* of the original function dictionary
 Compatibility notes
 ---------------------------------------------------------------
 
-Version 3.0 is a complete rewrite of the original implementation.
-It is mostly compatible with the past, a part for a few differences.
+Version 3.2 is the first version of the ``decorator`` module to officially
+support Python 3. Actually, the module has supported Python 3 from
+the beginning, via the ``2to3`` conversion tool, but this step has
+been now integrated in the build process, thanks to the distribute_
+project, the Python 3-compatible replacement of easy_install. 
+The hard work (for me) has been converting the documentation and the
+doctests.  This has been possible only now that docutils_ and pygments_
+have been ported to Python 3. 
 
-First of all, the utilites ``get_info`` and ``new_wrapper``, available
-in the 2.X versions, have been deprecated and they will be removed
-in the future. For the moment, using them raises a ``DeprecationWarning``.
-Incidentally, the functionality has been implemented through a
-decorator which makes a good example for this documentation:
+The ``decorator`` module *per se* does not contain any change, apart
+from the removal of the functions ``get_info`` and ``new_wrapper``,
+which have been deprecated for years. ``get_info`` has been removed
+since it was little used and since it had to be changed anyway to work
+with Python 3.0; ``new_wrapper`` has been removed since it was
+useless: its major use case (converting signature changing decorators
+to signature preserving decorators) has been subsumed by
+``decorator_apply`` and the other use case can be managed with the
+``FunctionMaker``.
 
-$$deprecated
-
-``get_info`` has been removed since it was little used and since it had
-to be changed anyway to work with Python 3.0; ``new_wrapper`` has been
-removed since it was useless: its major use case (converting
-signature changing decorators to signature preserving decorators)
-has been subsumed by ``decorator_apply``
-and the other use case can be managed with the ``FunctionMaker``.
+There are a few changes in the documentation: I removed the 
+``decorator_factory`` example, which was confusing some of my users,
+and I removed the part about exotic signatures in the Python 3
+documentation, since Python 3 does not support them.
+Notice that there is no support for Python 3 `function annotations`_ 
+since it seems premature at the moment, when most people are
+still using Python 2.X.
 
 Finally ``decorator`` cannot be used as a class decorator and the
 `functionality introduced in version 2.3`_ has been removed. That
 means that in order to define decorator factories with classes you
 need to define the ``__call__`` method explicitly (no magic anymore).
-Since there is
-an easy way to define decorator factories by using ``decorator_factory``,
-there is less need to use classes to implement decorator factories.
-
 All these changes should not cause any trouble, since they were
 all rarely used features. Should you have any trouble, you can always
 downgrade to the 2.3 version.
 
-The examples shown here have been tested with Python 2.5. Python 2.4
+The examples shown here have been tested with Python 2.6. Python 2.4
 is also supported - of course the examples requiring the ``with``
-statement will not work there. Python 2.6 works fine, but if you
-run the examples here in the interactive interpreter
+statement will not work there. Python 2.5 works fine, but if you
+run the examples in the interactive interpreter
 you will notice a few differences since
 ``getargspec`` returns an ``ArgSpec`` namedtuple instead of a regular
 tuple. That means that running the file
-``documentation.py`` under Python 2.5 will a few errors, but
-they are not serious. Python 3.0 is kind of supported too.
-Simply run the script ``2to3`` on the module
-``decorator.py`` and you will get a version of the code running
-with Python 3.0 (at least, I did some simple checks and it seemed
-to work). However there is no support for `function annotations`_ yet
-since it seems premature at this moment (most people are
-still using Python 2.5).
+``documentation.py`` under Python 2.5 will print a few errors, but
+they are not serious. 
 
 .. _functionality introduced in version 2.3: http://www.phyast.pitt.edu/~micheles/python/documentation.html#class-decorators-and-decorator-factories
 .. _function annotations: http://www.python.org/dev/peps/pep-3107/
-
+.. _distribute: http://packages.python.org/distribute/
+.. _docutils: http://docutils.sourceforge.net/
+.. _pygments: http://pygments.org/
 
 LICENCE
 ---------------------------------------------
@@ -866,10 +826,6 @@ today = time.strftime('%Y-%m-%d')
 
 __doc__ = __doc__.replace('$VERSION', VERSION).replace('$DATE', today)
 
-def decorator_factory(decfac): # partial is functools.partial
-    "decorator_factory(decfac) returns a one-parameter family of decorators"
-    return partial(lambda df, param: decorator(partial(df, param)), decfac)
-
 def decorator_apply(dec, func):
     """
     Decorate a function by preserving the signature even if dec 
@@ -942,7 +898,7 @@ def identity_dec(func):
 @identity_dec
 def example(): pass
 
-def memoize25(func):
+def memoize_uw(func):
     func.cache = {}
     def memoize(*args, **kw):
         if kw: # frozenset is used to ensure hashability
@@ -972,19 +928,20 @@ def _memoize(func, *args, **kw):
 def memoize(f):
     f.cache = {}
     return decorator(_memoize, f)
-    
-@decorator_factory
-def blocking(not_avail, f, *args, **kw):
-    if not hasattr(f, "thread"): # no thread running
-        def set_result(): f.result = f(*args, **kw)
-        f.thread = threading.Thread(None, set_result)
-        f.thread.start()
-        return not_avail
-    elif f.thread.isAlive():
-        return not_avail
-    else: # the thread is ended, return the stored result
-        del f.thread
-        return f.result
+
+def blocking(not_avail):
+    def blocking(f, *args, **kw):
+        if not hasattr(f, "thread"): # no thread running
+            def set_result(): f.result = f(*args, **kw)
+            f.thread = threading.Thread(None, set_result)
+            f.thread.start()
+            return not_avail
+        elif f.thread.isAlive():
+            return not_avail
+        else: # the thread is ended, return the stored result
+            del f.thread
+            return f.result
+    return decorator(blocking)
 
 class User(object):
     "Will just be able to see a page"
@@ -1001,16 +958,17 @@ def get_userclass():
 class PermissionError(Exception):
     pass
 
-@decorator_factory
-def restricted(user_class, func, *args, **kw):
-    "Restrict access to a given class of users"
-    userclass = get_userclass()
-    if issubclass(userclass, user_class):
-        return func(*args, **kw)
-    else:
-        raise PermissionError(
-            '%s does not have the permission to run %s!'
-            % (userclass.__name__, func.__name__))
+def restricted(user_class):
+    def restricted(func, *args, **kw):
+        "Restrict access to a given class of users"
+        userclass = get_userclass()
+        if issubclass(userclass, user_class):
+            return func(*args, **kw)
+        else:
+            raise PermissionError(
+                '%s does not have the permission to run %s!'
+                % (userclass.__name__, func.__name__))
+    return decorator(restricted)
 
 class Action(object):
     """
@@ -1077,8 +1035,7 @@ def fact(n): # this is not tail-recursive
     if n == 0: return 1
     return n * fact(n-1)
 
-
-def atest_for_pylons():
+def a_test_for_pylons():
     """
     In version 3.1.0 decorator(caller) returned a nameless partial
     object, thus breaking Pylons. That must not happen again.
@@ -1086,10 +1043,10 @@ def atest_for_pylons():
     >>> decorator(_memoize).__name__
     '_memoize'
 
-    Here is another bug of version 3.1.1 to avoid:
+    Here is another bug of version 3.1.1 missing the docstring to avoid:
 
-    >>> deprecated.__doc__
-    'A decorator for deprecated functions'
+    >>> factorial.__doc__
+    'The good old factorial'
     """
 
 if __name__ == '__main__':
